@@ -95,7 +95,8 @@ int Lexer::nextTokenKind()
         createNewline(m_curpos);
         return 0;
     }
-    QChar* it = m_content.data();
+
+    const QChar* it = m_content.constData();
     it += m_curpos;
     m_tokenBegin = m_curpos;
     switch (state()) {
@@ -344,8 +345,13 @@ int Lexer::nextTokenKind()
                     }
                 }
             } else if ((it + 1)->unicode() == '=') {
-                m_curpos++;
-                token = Parser::Token_IS_SMALLER_OR_EQUAL;
+                if ((it + 2)->unicode() == '>') {
+                    m_curpos += 2;
+                    token = Parser::Token_SPACESHIP;
+                } else {
+                    m_curpos++;
+                    token = Parser::Token_IS_SMALLER_OR_EQUAL;
+                }
             } else if ((it + 1)->unicode() == '>') {
                 m_curpos++;
                 token = Parser::Token_IS_NOT_EQUAL;
@@ -382,6 +388,9 @@ int Lexer::nextTokenKind()
                 token = Parser::Token_CLOSE_TAG;
                 m_curpos++;
                 while (state() != HtmlState) popState();
+            } else if ((it + 1)->unicode() == '?') {
+                token = Parser::Token_NULL_COALESCE;
+                m_curpos++;
             } else {
                 token = Parser::Token_QUESTION;
             }
@@ -421,7 +430,7 @@ int Lexer::nextTokenKind()
                             break;
                         } else {
                             // lookahead to check whether this doc comment spans multiple lines
-                            QChar* it2 = it + 1;
+                            const QChar* it2 = it + 1;
                             int pos = m_curpos + 1;
                             while ( pos < m_contentSize && (it2)->isSpace() && (it2)->unicode() != '\n' ) {
                                 ++it2;
@@ -544,6 +553,9 @@ int Lexer::nextTokenKind()
             if ((it + 1)->unicode() == '=') {
                 m_curpos++;
                 token = Parser::Token_CONCAT_ASSIGN;
+            } else if ((it + 1)->unicode() == '.' && (it + 2)->unicode() == '.') {
+                 m_curpos = m_curpos + 2;
+               token = Parser::Token_ELLIPSIS;
             } else {
                 token = Parser::Token_CONCAT;
             }
@@ -692,7 +704,7 @@ int Lexer::nextTokenKind()
             } else if (name.compare(QLatin1String("return"), Qt::CaseInsensitive) == 0) {
                 token = Parser::Token_RETURN;
             } else if (name.compare(QLatin1String("static"), Qt::CaseInsensitive) == 0) {
-                QChar* lookAhead = it;
+                const QChar* lookAhead = it;
                 int pos = m_curpos;
                 while (pos < m_contentSize && lookAhead->isSpace()) {
                     ++lookAhead;
@@ -782,6 +794,18 @@ int Lexer::nextTokenKind()
                 token = Parser::Token_NAMESPACE_C;
             } else if (name.compare(QLatin1String("callable"), Qt::CaseInsensitive) == 0) {
                 token = Parser::Token_CALLABLE;
+            } else if (name.compare(QLatin1String("iterable"), Qt::CaseInsensitive) == 0) {
+                token = Parser::Token_ITERABLE;
+            } else if (name.compare(QLatin1String("bool"), Qt::CaseInsensitive) == 0) {
+                token = Parser::Token_BOOL;
+            } else if (name.compare(QLatin1String("float"), Qt::CaseInsensitive) == 0) {
+                token = Parser::Token_FLOAT;
+            } else if (name.compare(QLatin1String("int"), Qt::CaseInsensitive) == 0) {
+                token = Parser::Token_INT;
+            } else if (name.compare(QLatin1String("string"), Qt::CaseInsensitive) == 0) {
+                token = Parser::Token_STRING_TYPE;
+            } else if (name.compare(QLatin1String("void"), Qt::CaseInsensitive) == 0) {
+                token = Parser::Token_VOID;
             } else {
                 token = Parser::Token_STRING;
             }
@@ -974,7 +998,7 @@ qint64 Lexer::tokenEnd() const
     return m_tokenEnd;
 }
 
-bool Lexer::isHereNowDocEnd(QChar* it)
+bool Lexer::isHereNowDocEnd(const QChar* it)
 {
     int identiferLen = m_hereNowDocIdentifier.length();
     QString lineStart;
@@ -992,7 +1016,7 @@ bool Lexer::isHereNowDocEnd(QChar* it)
 }
 
 //used for strings, to check if " is escaped (\" is, \\" not)
-bool Lexer::isEscapedWithBackslash(QChar* it, int curPos, int startPos)
+bool Lexer::isEscapedWithBackslash(const QChar* it, int curPos, int startPos)
 {
     int cnt = 0;
     it--;
@@ -1003,9 +1027,9 @@ bool Lexer::isEscapedWithBackslash(QChar* it, int curPos, int startPos)
     return (cnt % 2) == 1;
 }
 
-bool Lexer::processVariable(QChar* it)
+bool Lexer::processVariable(const QChar* it)
 {
-    QChar* c2 = it + 1;
+    const QChar* c2 = it + 1;
     if (it->unicode() == '$' && (isValidVariableIdentifier(c2) && !c2->isDigit())) {
         it++;
         m_curpos++;
@@ -1020,7 +1044,7 @@ bool Lexer::processVariable(QChar* it)
         return false;
     }
 }
-bool Lexer::isValidVariableIdentifier(QChar* it)
+bool Lexer::isValidVariableIdentifier(const QChar* it)
 {
     return it->isLetter() || it->isDigit() || it->unicode() == '_' || it->unicode() > 0x7f;
 }
